#!/usr/bin/env scheme-script
;; -*- mode: scheme; coding: utf-8 -*- !#
;; Copyright © 2018 Göran Weinholt <goran@weinholt.se>
;; SPDX-License-Identifier: MIT
#!r6rs

(import
  (rnrs (6))
  (srfi :64 testing)
  (semver versions)
  (semver ranges))

(test-begin "range-parse")
(test-equal "=1.2.3-beta.0+001" (semver-range->string (string->semver-range "1.2.3-beta.0+001")))
(test-equal "=1.2.3-beta.0" (semver-range->string (string->semver-range "1.2.3-beta.0")))
(test-equal "=1.2.3" (semver-range->string (string->semver-range "1.2.3")))
(test-equal "=1.2" (semver-range->string (string->semver-range "1.2")))
(test-equal "=1" (semver-range->string (string->semver-range "1")))
(test-equal "=1.2.*" (semver-range->string (string->semver-range "1.2.*")))
(test-equal "=1.*.*" (semver-range->string (string->semver-range "1.*.*")))
(test-equal "=1.*" (semver-range->string (string->semver-range "1.*")))
(test-equal "=1.2.*" (semver-range->string (string->semver-range "1.2.x")))
(test-equal "=1.*.*" (semver-range->string (string->semver-range "1.x.x")))
(test-equal "=1.*" (semver-range->string (string->semver-range "1.x")))
(test-equal "*" (semver-range->string (string->semver-range "")))
(test-equal "*" (semver-range->string (string->semver-range "*")))
(test-equal "=1" (semver-range->string (string->semver-range "1")))
(test-equal "=1 || =2" (semver-range->string (string->semver-range "1 || 2")))
(test-equal "=1 || =2 || =3" (semver-range->string (string->semver-range "1 || 2 || 3")))
(test-equal ">1 <2" (semver-range->string (string->semver-range ">1 <2")))
(test-equal "~1.2 ^1.3" (semver-range->string (string->semver-range "~1.2 ^1.3")))
(test-equal "1.2.3 - 2.3.4" (semver-range->string (string->semver-range "1.2.3 - 2.3.4")))
(test-equal ">=*" (semver-range->string (string->semver-range ">=x")))
(test-end)

(define (desugar x)
  (semver-range->string (semver-range-desugar (string->semver-range x))))

(test-begin "range-desugar-hyphen")
(test-equal ">=1.2.3 <=2.3.4" (desugar "1.2.3 - 2.3.4"))
(test-equal ">=1.2.0 <=2.3.4" (desugar "1.2 - 2.3.4"))
(test-equal ">=1.0.0 <=2.3.4" (desugar "1 - 2.3.4"))
(test-equal ">=1.2.3 <2.4.0" (desugar "1.2.3 - 2.3"))
(test-equal ">=1.2.3 <3.0.0" (desugar "1.2.3 - 2"))
(test-end)

(test-begin "range-desugar-x")
(test-equal ">=0.0.0" (desugar ""))
(test-equal ">=1.0.0 <2.0.0" (desugar "1.x"))
(test-equal ">=1.2.0 <1.3.0" (desugar "1.2.x"))
(test-equal ">=0.0.0" (desugar "*"))
(test-equal ">=1.0.0 <2.0.0" (desugar "1"))
(test-equal ">=1.2.0 <1.3.0" (desugar "1.2"))
(test-end)

(test-begin "range-desugar-tilde")
(test-equal ">=1.2.3 <1.3.0" (desugar "~1.2.3"))
(test-equal ">=1.2.0 <1.3.0" (desugar "~1.2"))
(test-equal ">=1.0.0 <2.0.0" (desugar "~1"))
(test-equal ">=0.2.3 <0.3.0" (desugar "~0.2.3"))
(test-equal ">=0.2.0 <0.3.0" (desugar "~0.2"))
(test-equal ">=0.0.0 <1.0.0" (desugar "~0"))
(test-equal ">=1.2.3-beta.2 <1.3.0" (desugar "~1.2.3-beta.2"))
(test-end)

(test-begin "range-desugar-caret")
(test-equal ">=1.2.3 <2.0.0" (desugar "^1.2.3"))
(test-equal ">=0.2.3 <0.3.0" (desugar "^0.2.3"))
(test-equal ">=0.0.3 <0.0.4" (desugar "^0.0.3"))
(test-equal ">=1.2.3-beta.2 <2.0.0" (desugar "^1.2.3-beta.2"))
(test-equal ">=0.0.3-beta <0.0.4"   (desugar "^0.0.3-beta"))
(test-equal ">=1.2.0 <2.0.0" (desugar "^1.2.x"))
(test-equal ">=0.0.0 <0.1.0" (desugar "^0.0.x"))
(test-equal ">=0.0.0 <0.1.0" (desugar "^0.0"))
(test-equal ">=1.0.0 <2.0.0" (desugar "^1.x"))
(test-equal ">=0.0.0 <1.0.0" (desugar "^0.x"))
(test-end)

(test-begin "range-desugar-misc")       ;slightly based on observed behavior
(test-equal ">=0.0.0" (desugar "<=x"))
(test-equal ">=0.0.0" (desugar ">=x"))
(test-equal ">=0.0.0" (desugar "=x"))
(test-equal "<0.0.0" (desugar "<x"))    ;hmm.
(test-equal ">=1.0.0" (desugar ">=1.x"))
(test-equal ">=1.2.0" (desugar ">=1.2.x"))
(test-equal ">=1.2.0" (desugar ">=1.2.x-p+b"))
(test-equal ">=1.2.3-p+b" (desugar ">=1.2.3-p+b"))
(test-equal ">=1.0.0 <2.0.0 >=1.0.0 <2.0.0" (desugar "1 1"))
(test-equal ">=1.0.0 <2.0.0 || >=2.0.0 <3.0.0" (desugar "1 || 2"))
(test-end)

(test-begin "range-match")
(test-assert (semver-in-range? "1.2.7" ">=1.2.7"))
(test-assert (semver-in-range? "1.2.8" ">=1.2.7"))
(test-assert (semver-in-range? "2.5.3" ">=1.2.7"))
(test-assert (semver-in-range? "1.3.9" ">=1.2.7"))
(test-assert (not (semver-in-range? "1.2.6" ">=1.2.7")))
(test-assert (not (semver-in-range? "1.1.0" ">=1.2.7")))
(test-assert (semver-in-range? "1.2.7" ">=1.2.7 <1.3.0"))
(test-assert (semver-in-range? "1.2.8" ">=1.2.7 <1.3.0"))
(test-assert (semver-in-range? "1.2.99" ">=1.2.7 <1.3.0"))
(test-assert (not (semver-in-range? "1.2.6" ">=1.2.7 <1.3.0")))
(test-assert (not (semver-in-range? "1.3.0" ">=1.2.7 <1.3.0")))
(test-assert (not (semver-in-range? "1.1.0" ">=1.2.7 <1.3.0")))
(test-assert (semver-in-range? "1.2.7" "1.2.7 || >=1.2.9 <2.0.0"))
(test-assert (semver-in-range? "1.2.9" "1.2.7 || >=1.2.9 <2.0.0"))
(test-assert (semver-in-range? "1.4.6" "1.2.7 || >=1.2.9 <2.0.0"))
(test-assert (not (semver-in-range? "1.2.8" "1.2.7 || >=1.2.9 <2.0.0")))
(test-assert (not (semver-in-range? "2.0.0" "1.2.7 || >=1.2.9 <2.0.0")))
(test-assert (not (semver-in-range? "1.2.10" "1.2 <1.2.9 || >2.0.0")))
(test-assert (semver-in-range? "1.2.8" "1.2 <1.2.9 || >2.0.0"))
(test-end)

(test-begin "range-match-pre")
(test-assert (semver-in-range? "1.2.3-alpha.7" ">1.2.3-alpha.3"))
(test-assert (not (semver-in-range? "3.4.5-alpha.9" ">1.2.3-alpha.3")))
(test-assert (semver-in-range? "3.4.5" ">1.2.3-alpha.3"))
(test-assert (semver-in-range? "1.2.3-beta.4" "~1.2.3-beta.2"))
(test-assert (not (semver-in-range? "1.2.4-beta.2" "~1.2.3-beta.2")))
(test-assert (semver-in-range? "1.2.3-beta.4" "^1.2.3-beta.2"))
(test-assert (not (semver-in-range? "1.2.4-beta.2" "^1.2.3-beta.2")))
(test-assert (semver-in-range? "0.0.3-pr.2" "^0.0.3-beta"))
(test-assert (not (semver-in-range? "0.0.3-alpha" "^0.0.3-beta")))
(test-end)


(exit (if (zero? (test-runner-fail-count (test-runner-get))) 0 1))
